import pygame
import random
from road import generate_roads,generate_cam_pos
from base import Location
from utils import *




# 行人类定义
class Pedestrian:
    def __init__(self, x, y, road_network, target, type=0):
        self.position = [x, y]
        self.target_road_index = 0
        self.road_network = road_network
        self.speed = 0.2
        self.target = target
        self.type = type  # 0 表示普通行人，1 表示聚集人群
    def update(self):
        # 检查是否到达目标道路
        #self.speed = 0.2

        x_next = self.position[0] + self.speed * sign(self.target.x - self.position[0])
        # 如果下一刻的X位置不在道路上, 则保持当前位置不变
        new_position = [x_next, self.position[1]]
        if(check_pos_in_roads(new_position, self.road_network) == False):
            x_next = self.position[0]

        y_next = self.position[1] + self.speed * sign(self.target.y - self.position[1])
        new_position = [x_next, y_next]
        # 如果下一刻的Y位置不在道路上, 则保持当前位置不变   
        if(check_pos_in_roads(new_position, self.road_network) == False):
            y_next = self.position[1]

        # if(self.target.y == self.position[1] and check_pos_x_in_roads(new_position[0], self.road_network) == False):
        #     # 尝试左右
        #     x_next = self.position[0] + self.speed * sign(self.target.x - self.position[0]) * 2
        #     new_position = [x_next, self.position[1]]       # 如果遇到x轴或者y轴到达目标，但是另外一侧的方向没用道路，则尝试尝试方向
      



        # 如果普通行人到达了终点则重新选择目标位置
        if(self.target.x == self.position[0] and self.target.y == self.position[1] and self.type == 0):
            
            # 随机选择一条道路
            road = random.choice(self.road_network)
            # 随机选择道路上的一个点
            x = random.randint(road.rect.left, road.rect.right)
            y = random.randint(road.rect.top, road.rect.bottom)
            self.target = Location(x, y)
            
        self.position = [x_next, y_next]

    

def generate_pedestrians(num_pedestrians, num_gather_pedestrians,road_network, target):
    pedestrians = []
    # 这里生成普通行人, 目标是随机的人
    for _ in range(num_pedestrians):
        # 随机选择一条道路
        road = random.choice(road_network)
        # 随机选择道路上的一个点
        x = random.randint(road.rect.left, road.rect.right)
        y = random.randint(road.rect.top, road.rect.bottom)

        xt = random.randint(road.rect.left, road.rect.right)
        yt = random.randint(road.rect.top, road.rect.bottom)

        pos = Location(xt, yt)
        # 创建行人
        pedestrian = Pedestrian(x, y, road_network, pos)
        pedestrians.append(pedestrian)
    
    for _ in range(num_gather_pedestrians):
        # 随机选择一条道路
        road = random.choice(road_network)
        # 随机选择道路上的一个点
        x = random.randint(road.rect.left, road.rect.right)
        y = random.randint(road.rect.top, road.rect.bottom)

        #target = Location(xt, yt)
        # 聚集人群的目标是固定的
        pedestrian = Pedestrian(x, y, road_network, target, type=1)
        pedestrians.append(pedestrian)

    return pedestrians

def distance(pos1, pos2):
    return ((pos1[0] - pos2[0]) ** 2 + (pos1[1] - pos2[1]) ** 2) ** 0.5


def main():
    # 初始化pygame
    pygame.init()

    # 定义类别颜色
    WHITE = (255, 255, 255)
    BLACK = (0, 0, 0)
    GRAY = (128, 128, 128)
    BLUE = (0, 0, 255)
    RED = (255, 0, 0)
    ORANGE = (179, 102, 73)

    MAP_WIDTH = 1400
    MAP_HEIGHT = 900
    # 设置仿真窗口大小
    size = (MAP_WIDTH, MAP_HEIGHT)
    # 创建窗口
    screen = pygame.display.set_mode(size)
    # 设置标题
    pygame.display.set_caption("行人沿城市道路行走与聚集仿真")


    #############   地图元素生成 #############

    # 生成道路网络
    roads = generate_roads(6, 6, MAP_WIDTH, MAP_HEIGHT)
    cam = generate_cam_pos(6, 6, MAP_WIDTH, MAP_HEIGHT)
    # 生成房屋
    houses = []
    house_img_index = []
    x_offset = 0
    y_offset = 0
    for x in range(0, MAP_WIDTH, 50):
        for y in range(0, MAP_HEIGHT, 50):
            house = pygame.Rect(x, y, 30, 30)
            if(rect_in_road(house, roads) == False):
                houses.append(house)
                index = random.randint(1, 10)
                house_img_index.append(index)
            

    # 生成聚集目标
    road = random.choice(roads)

    x = random.randint(road.rect.left, road.rect.right)
    y = random.randint(road.rect.top, road.rect.bottom)
    
    print(x, y)
    target = Location(x, y)

    # 创建一些行人,行人必须在路上
    pedestrians = generate_pedestrians(100, 200, roads, target)


    #############  贴图加载 ###############

    cam_img = pygame.image.load('assets/cam.png')
    new_size = (40, 40)
    cam_img = pygame.transform.scale(cam_img, new_size)


    
    house_img_list = []
    for i in range(1, 11):
        print('assets/house' + str(i) + '.png')
        house_img = pygame.image.load('assets/house' + str(i) + '.png')
        new_size = (60, 60)
        house_img = pygame.transform.scale(house_img, new_size)
        house_img_list.append(house_img)

    done = False
    clock = pygame.time.Clock()

    while not done:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                done = True

            pygame.draw.rect(screen, BLACK, house)

        for pedestrian in pedestrians:
            # 检查在附近的人数
            range_num = 30
            count = 0
            for other_pedestrian in pedestrians:
                if(pedestrian == other_pedestrian):
                    continue
                if(distance(pedestrian.position, other_pedestrian.position) < range_num):
                    count += 1
                
            # 如果周围有很多人，则降低速度(人群的聚集效应)
            if(count >= 3 and count < 5):
                pedestrian.speed = 0.1
            elif(count >= 5):
                pedestrian.speed = 0.08
            else:
                pedestrian.speed = 0.2

        # 移动行人
        for pedestrian in pedestrians:
            pedestrian.update()

        # 绘制所有内容
        screen.fill(WHITE)

        # for house in houses:
        #     # 根据预先生成的索引来选择房屋图片
        #     index = house_img_index[houses.index(house)]
        #     print(index)
        #     img = house_img_list[index - 1]
        #     screen.blit(img, (house.x - 10, house.y - 10))


        # 绘制马路
        for road in roads:
            pygame.draw.rect(screen, GRAY, road.rect)
        # 绘制行人
        for pedestrian in pedestrians:
            if(pedestrian.type == 1):
                pygame.draw.circle(screen, RED, pedestrian.position, 3)
            else:
                pygame.draw.circle(screen, BLUE, pedestrian.position, 3)
        # 绘制摄像头
        for cam_pos in cam:
            screen.blit(cam_img, (cam_pos[0] - 10, cam_pos[1] - 10))

        pygame.draw.circle(screen, RED, target.position, 12)

        pygame.display.flip()
        clock.tick(60)

    pygame.quit()

if __name__ == "__main__":
    main()